<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Zarif Safiullin <zaph.saph@gmail.com>
 * @package ow_plugins.qa.controllers
 * @since 1.0
 */
class QA_CTRL_View extends OW_ActionController
{

    public function index( $params )
    {

        $username = !empty($params['user']) ? $params['user'] : '';

        $id = $params['id'];

        $plugin = OW::getPluginManager()->getPlugin('qa');

        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'qa', 'main_menu_item');

        $service = QuestionService::getInstance();

        $userService = BOL_UserService::getInstance();
		
        $userId = OW::getUser()->getId();
        
        $this->assign('user', ((OW::getUser()->getId() !== null) ? $userService->findUserById(OW::getUser()->getId()) : null));

        $question = $service->findById($id);
        
		$this->assign('category', $category = QA_BOL_CategoryDao::getInstance()->findById($question->categoryId));
		
//        if(OW::getUser()->getId() != 0 && OW::getUser()->getId() != $question->authorId){
			$question->viewed_count++;
			QuestionService::getInstance()->save($question);
//        }
        
        if ( $question === null )
        {
            throw new Redirect404Exception();
        }

        if ($question->isDraft() && $question->authorId != OW::getUser()->getId())
        {
            throw new Redirect404Exception();
        }

        $question->description = BASE_CMP_TextFormatter::fromBBtoHtml($question->description);
        $question->setTitle( strip_tags($question->getTitle()) );

        if ( !OW::getUser()->isAuthorized('qa', 'view') )
        {
            $this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');

            return;
        }

        if ( ( OW::getUser()->isAuthenticated() && OW::getUser()->getId() != $question->getAuthorId() ) && !OW::getUser()->isAuthorized('qa', 'view') )
        {
            $this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');

            return;
        }

        /* Check privacy permissions */
        if ( $question->authorId != OW::getUser()->getId() && !OW::getUser()->isAuthorized('qa') )
        {
            $eventParams = array(
                'action' => 'qa_view_qa_questions',
                'ownerId' => $question->authorId,
                'viewerId' => OW::getUser()->getId()
            );

            OW::getEventManager()->getInstance()->call('privacy_check_permission', $eventParams);
        }
        /* */

        $parts = explode('<!--page-->', $question->getQuestion());

        $page = !empty($_GET['page']) ? $_GET['page'] : 1;

        $count = count($parts);

        if ( strlen($username) > 0 )
        {
            $author = $userService->findByUsername($username);
        }
        else
        {
            $author = $userService->findUserById($question->getAuthorId());
            $isAuthorExists = !empty($author);
            if ( $isAuthorExists )
            {
                $username = $author->getUsername();
            }
        }

        $this->assign('isAuthorExists', $isAuthorExists);
		$tagValue = "";
        if ( $isAuthorExists )
        {
            $displayName = $userService->getDisplayName($author->getId());

            $this->assign('username', $userService->getUserName($author->getId()));
            $this->assign('displayname', $displayName);

            $url = OW::getRouter()->urlForRoute('user-qa', array('user' => $username, 'list' => 'questions'));
			
            //$this->setPageHeading(OW::getLanguage()->text('qa', 'view_page_heading', array('url' => $url, 'name' => $displayName, 'questionTitle' => htmlspecialchars($question->getTitle()))));
            $this->setPageHeadingIconClass('ow_ic_write');

            OW::getDocument()->setTitle(OW::getLanguage()->text('qa', 'qa_question_title', array('question_title' => htmlspecialchars($question->getTitle()), 'display_name' => $displayName)));

            $question_body = UTIL_String::truncate($question->getQuestion(), 200, '...');
            $questionTagsArray = BOL_TagService::getInstance()->findEntityTags($question->getId(), 'qa-question');
            $questionTags = "";
			
            foreach ( $questionTagsArray as $tag )
            {
                $questionTags .= $tag->label . ", ";
				$tagValue .='<a href="' . OW::getRouter()->urlForRoute('qa.list', array('list'=>'browse-by-tag')) . "?tag={$tag->label}" . "\">{$tag->label}</a>, ";
            }
            $questionTags = substr($questionTags, 0, -2);
            OW::getDocument()->setDescription(OW::getLanguage()->text('qa', 'qa_question_description', array('question_body' => htmlspecialchars(strip_tags($question_body)), 'tags' => htmlspecialchars($questionTags))));
            OW::getDocument()->setKeywords(OW::getLanguage()->text('nav', 'page_default_keywords').", ".$questionTags);
        }
	
		 $this->assign('ownerUrl', $url);
		 $this->assign('tagValue',  mb_substr($tagValue, 0, mb_strlen($tagValue) - 2));

        $info = array(
            'dto' => $question,
            'text' => $parts[$page - 1]
        );

        $this->assign('info', $info);

        if ( $isAuthorExists )
        {
            //qa navigation
            $prev = $service->findAdjacentUserQuestion($author->getId(), $question->getId(), 'prev');
            $next = $service->findAdjacentUserQuestion($author->getId(), $question->getId(), 'next');

            if ( !empty($prev) )
            {
                $prevUser = $userService->findUserById($prev->getAuthorId());
            }

            if ( !empty($next) )
            {
                $nextUser = $userService->findUserById($next->getAuthorId());
            }

            $this->assign('adjasentUrl',
                array(
                    'next' => (!empty($nextUser) ) ? OW::getRouter()->urlForRoute('user-question', array('id' => $next->getId(), 'user' => $nextUser->getUsername())) : '',
                    'prev' => (!empty($prevUser) ) ? OW::getRouter()->urlForRoute('user-question', array('id' => $prev->getId(), 'user' => $prevUser->getUsername())) : '',
                    'index' => OW::getRouter()->urlForRoute('user-qa', array('user' => $author->getUsername(), 'list' => 'questions'))
                )
            );
        }
        else
        {
            $this->assign('adjasentUrl', null);
        }
        //~qa navigation
        //toolbar

        $tb = array();

        $toolbarEvent = new BASE_CLASS_EventCollector('qa.collect_question_toolbar_items', array(
            'questionId' => $question->id,
            'questionDto' => $question
        ));

        OW::getEventManager()->trigger($toolbarEvent);

        foreach ( $toolbarEvent->getData() as $toolbarItem )
        {
            array_push($tb, $toolbarItem);
        }

        if ( OW::getUser()->isAuthenticated() && ( $question->getAuthorId() != OW::getUser()->getId() ) )
        {
            $js = UTIL_JsGenerator::newInstance()
                ->jQueryEvent('#qa_question_toolbar_flag', 'click', UTIL_JsGenerator::composeJsString('OW.flagContent({$entity}, {$id}, {$title}, {$href}, "qa+flags");',
                            array('entity' => 'qa_question', 'id' => $question->getId(), 'title' => htmlspecialchars(json_encode($question->getTitle())), 'href' => OW::getRouter()->urlForRoute('user-question', array('id' => $question->getId()) )  )));

            OW::getDocument()->addOnloadScript($js, 1001);

            $tb[] = array(
                'label' => OW::getLanguage()->text('base', 'flag'),
                'href' => 'javascript://',
                'id' => 'qa_question_toolbar_flag'
            );
        }
        if ( OW::getUser()->isAuthenticated() && ( OW::getUser()->getId() == $question->getAuthorId() || BOL_AuthorizationService::getInstance()->isModerator(OW::getUser()->getId())) )
        {
            $tb[] = array(
                'href' => OW::getRouter()->urlForRoute('question-save-edit', array('id' => $question->getId())),
                'label' => OW::getLanguage()->text('qa', 'toolbar_edit')
            );

            $tb[] = array(
                'href' => OW::getRouter()->urlFor('QA_CTRL_Save', 'delete', array('id' => $question->getId())),
                'click' => "return confirm('" . OW::getLanguage()->text('base', 'are_you_sure') . "');",
                'label' => OW::getLanguage()->text('qa', 'toolbar_delete')
            );
        }

        $this->assign('tb', $tb);
        //~toolbar

        $paging = new BASE_CMP_Paging($page, $count, $count);

        //<ARCHIVE-NAVIGATOR>


        $this->assign('paging', $paging->render());

        //</ARCHIVE-NAVIGATOR>
        if ( $isAuthorExists )
        {
            $this->assign('author', $author);
        }

        $this->assign('isModerator', OW::getUser()->isAuthorized('qa'));
        if ( $isAuthorExists )
        {
            $this->assign('userQaUrl', OW::getRouter()->urlForRoute('user-qa', array('user' => $author->getUsername(), 'list' => 'questions')));
        }

        /* Check comments privacy permissions */
        $allowAnswer = true;
        if ( $question->authorId != OW::getUser()->getId() && !OW::getUser()->isAuthorized('qa') )
        {
            $eventParams = array(
                'action' => 'qa_comment_qa_questions',
                'ownerId' => $question->authorId,
                'viewerId' => OW::getUser()->getId()
            );

            try
            {
                OW::getEventManager()->getInstance()->call('privacy_check_permission', $eventParams);
            }
            catch ( RedirectException $ex )
            {
                $allowAnswer = false;
            }
        }
        /* */
        //get answers list

        //answer form
         $this->addComponent('answerForm', new QA_CMP_AnswersForm(array('questionId' => $id, 'allowAnswer' => $allowAnswer)));
        $this->assign('totalAnswers', $question->getAnswerTotal());
		
        $this->assign('avatarUrl', '');
		$this->assign('allowAnswer', $allowAnswer);
		
        $tagCloud = new BASE_CMP_EntityTagCloud('qa-question', OW::getRouter()->urlForRoute('qa.list', array('list'=>'browse-by-tag')));

        $tagCloud->setEntityId($question->getId());

        $rateInfo = new BASE_CMP_Rate('qa', 'qa-question', $question->getId(), $question->getAuthorId());

        $this->addComponent('rate', $rateInfo);

        $this->addComponent('tagCloud', $tagCloud);
		
        $this->addComponent('categoryWidget', new QA_CMP_Category(array('cat' => $question->categoryId)));
        
        $this->addComponent('searchWidget', new QA_CMP_SearchForm());
        
		OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('qa')->getStaticCssUrl() . 'style.css');
		OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('qa')->getStaticCssUrl() . 'default.css');
		if($allowAnswer){
			OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('qa')->getStaticJsUrl() . 'answer-reply.js');
		}
		
		$deleteLink = OW::getRouter()->urlFor('QA_CTRL_Answer', 'delete');
		$rateLink = OW::getRouter()->urlFor('QA_CTRL_Answer', 'rate');
		$confirm = OW::getLanguage()->text('base', 'are_you_sure');
		$canGuessRate = ($userId)? 1 : 0;
		$rate_cmp_auth_error_message  = OW::getLanguage()->text('base', 'rate_cmp_auth_error_message');
		$rate_cmp_owner_cant_rate_error_message = OW::getLanguage()->text('base', 'rate_cmp_owner_cant_rate_error_message');
		$edit_answer = OW::getLanguage()->text('qa', 'edit_answer');
		$content = <<<CONTENT
$.each($('.btn-answer-edit'), function(){
    $(this).click(function(){
        var answer_id = $(this).attr("rel");
            window.edit_answer_floatbox = OW.ajaxFloatBox(
                "QA_CMP_EditAnswer",
                { answerId : answer_id },
                { width : 700, iconClass : "ow_ic_edit", title : '{$edit_answer}',
                    onLoad : function() {
                        var textarea = $("#contentEdit").get(0);
                        textarea.htmlarea();
                        textarea.htmlareaRefresh();

                        owForms['answer-edit-form'].bind("success", function(data){
                            window.edit_answer_floatbox.close();
                            if ( data.link ) { window.location = data.link };
                        });
                    }
                }
            );
    });
});


$.each($('.btn-answer-delete'), function(){
    $(this).click(function(){
        var answer_id = $(this).attr("rel");
		if ( confirm('{$confirm}') ) {
               window.location = '{$deleteLink}?id=' + answer_id;
            }
    });
});

$.each($('.thumbs .canVote'), function(){
	$(this).unbind('click').click(clickfunction = function(){
		var canGuessRate = {$canGuessRate};
		if(!canGuessRate) {OW.error("{$rate_cmp_auth_error_message}"); return;}
        var answer_id = $(this).attr("rel");
        var owner_id = $(this).attr("owner-id");
        if(owner_id == {$userId}) {OW.error("{$rate_cmp_owner_cant_rate_error_message}"); return;}
		var rating = ($(this).hasClass("up"))? 1 : -1;
		$.ajax( {
                    url : '{$rateLink}',
                    type : 'POST',
                    data : {answerId : answer_id,
    				 		rating: rating
    						},
                    dataType : 'json',
                    success : function(data) {
                        if ( data.result == true ) {
                            OW.info(data.msg);
                            
                            var answer = data.obj;
                            if(rating == 1){
                            	$('#comment-'+ answer_id).find('.meta-left').first().removeClass('voted_down').addClass('voted_up');
                            }else{
    							$('#comment-'+ answer_id).find('.meta-left').first().removeClass('voted_up').addClass('voted_down');
    						}	
    						
    						if(answer.rating > 0){
    							$('#comment-'+ answer_id).find('.voting .rating').first().removeClass('low').removeClass('mid').addClass('high');
    						}else if(answer.rating < 0){
    							$('#comment-'+ answer_id).find('.voting .rating').first().removeClass('high').removeClass('mid').addClass('low');
    						}else{
    							$('#comment-'+ answer_id).find('.voting .rating').first().removeClass('high').removeClass('low').addClass('mid');
    						}
    						
    						var perfix = (answer.rating > 0)? '+':'';
    						$('#comment-'+ answer_id).find('.voting .rating .percent span').first().html(perfix + answer.rating);
    						
    						$('#comment-'+ answer_id).find('.voting .up').first().removeClass('canVote');
    						$('#comment-'+ answer_id).find('.voting .down').first().removeClass('canVote');
    						if(rating == 1){
    							 $('#comment-'+ answer_id).find('.voting .down').first().addClass('canVote');
    						}else if(rating == -1){
    							 $('#comment-'+ answer_id).find('.voting .up').first().addClass('canVote');
    						}  
    						
    						$('#comment-'+ answer_id).find('.voting .vote_count').first().html(data.totalVote);
    						
    						//bindUI again
    						$.each($('.thumbs .canVote'), function(){
								$(this).unbind('click').click(clickfunction)
							});	
                        } else if ( data.error != undefined ) {
                            OW.warning(data.error);
                        }
                    }
                });
    });    
});
CONTENT;

    OW::getDocument()->addOnloadScript($content);
		
        //~ additional components
    }
}